<?php

namespace EventPortal\Domain\Form;

use EventPortal\Domain\Component\Section;
use EventPortal\Domain\Event\Form\Models\Form as FormModel;
use EventPortal\Domain\Event\Form\Models\FormStep;
use EventPortal\Domain\Event\Form\Models\FormStepSection;
use EventPortal\Domain\Event\Form\Models\Question;
use EventPortal\Domain\Event\Form\Models\QuestionValidation;
use Illuminate\Support\Arr;

class FormRegistry
{
    /** @var array<string> */
    private $forms = [];

    public function make(string $formName): Form
    {
        if (Arr::has($this->forms, $formName)) {
            return $this->forms[$formName];
        }

        $this->forms[$formName] = new Form($formName);

        return $this->forms[$formName];
    }

    public function makeFromDatabase(FormModel $form, ?FormStep $formStep = null): Form
    {
        $step = $formStep ?? $form->steps->first();

        if (Arr::has($this->forms, $step->name)) {
            return $this->forms[$step->name];
        }

        $step->loadMissing([
            'sections.questions.options',
            'sections.questions.validations',
        ]);

        $this->forms[$step->name] = new Form($step->name);
        $this->forms[$step->name]->fromDatabase();
        $this->forms[$step->name]->addFields(static function () use ($step): array {
            return $step->sections->map(function (FormStepSection $section): Section {
                return Section::make($section->name)->fields(
                    $section->questions->map(function (Question $question) {
                        return self::makeQuestionComponent($question);
                    })->all()
                );
            })->all();
        });

        return $this->forms[$step->name];
    }

    /**
     * @return mixed
     */
    private static function makeQuestionComponent(Question $question)
    {
        $component = $question->type->getBuilderType()::make($question->question)->scopeTo('formData');

        $options = $question->options;

        if ($options->isEmpty() === false) {
            $options = $options
                ->map
                ->asOption()
                ->all();

            $component->options($options);
        }

        $validations = $question->validations;

        if ($validations->isEmpty() === false) {
            $validations = $validations
                ->map(static function (QuestionValidation $validation): string {
                    return $validation->rule;
                })
                ->all();

            $component->rules($validations);
        }

        if ($question->isGdprSensitive()) {
            $component->gdpr();
        }

        if ($question->hasHelpText()) {
            $component->helpText($question->help_text);
        }

        if ($question->hasPlaceholder()) {
            $component->placeholder($question->placeholder);
        }

        return $component;
    }
}
